from setuptools import setup, find_packages
from os.path import abspath, dirname, join

# Fetches the content from README.md
# This will be used for the "long_description" field.
README_MD = open(join(dirname(abspath(__file__)), "README.md")).read()

setup(
    name="trueupdate",
    version="2.1.4",

    # The packages that constitute your project.
    # For my project, I have only one - "pydash".
    # Either you could write the name of the package, or
    # alternatively use setuptools.findpackages()
    #
    # If you only have one file, instead of a package,
    # you can instead use the py_modules field instead.
    # EITHER py_modules OR packages should be present.
    packages=find_packages(),
    
    entry_points = {
        'console_scripts': ['trueupdate=trueupdate.command_line:main'],
    },

    # The description that will be shown on PyPI.
    # Keep it short and concise
    # This field is OPTIONAL
    description="An Automatic and Bulk update utility for TrueNAS SCALE Apps",

    # The content that will be shown on your project page.
    # In this case, we're displaying whatever is there in our README.md file
    # This field is OPTIONAL
    long_description=README_MD,

    # Now, we'll tell PyPI what language our README file is in.
    # In my case it is in Markdown, so I'll write "text/markdown"
    # Some people use reStructuredText instead, so you should write "text/x-rst"
    # If your README is just a text file, you have to write "text/plain"
    # This field is OPTIONAL
    long_description_content_type="text/markdown",

    # The url field should contain a link to a git repository, the project's website
    # or the project's documentation. I'll leave a link to this project's Github repository.
    # This field is OPTIONAL
    url="https://github.com/truecharts/trueupdate",

    # The author name and email fields are self explanatory.
    # These fields are OPTIONAL
    author_name="truecharts",
    author_email="into@truecharts.org",

    # For additional fields, check:
    # https://github.com/pypa/sampleproject/blob/master/setup.py
)



import subprocess
import sys
import argparse
import time

class Chart(object):
    def __setattr__(self, name, value):
        if name == 'status':
            value = value.casefold()
        if 'update_available' in name:
            value = True if value.casefold() == "true" else False
        super(Chart, self).__setattr__(name, value)

    def new_attr(self, attr):
        setattr(self, attr, attr)

INSTALLED_CHARTS = []

def parse_headers(charts: str):
    for line in charts.split("\n"):
        if line.startswith("+-"):
            continue
        if "name" in line.casefold():
            return [col.strip() for col in line.casefold().strip().split("|") if col and col != ""]

def parse_charts(charts: str):
    headers = parse_headers(charts)
    table_data = charts.split("\n")[3:-2:]  # Skip the header part of the table
    for row in table_data:
        row = [section.strip() for section in row.split("|") if section and section != ""]
        chart = Chart()
        for item in zip(headers, row):
            setattr(chart, item[0], item[1])
        INSTALLED_CHARTS.append(chart)

def check_semver(current: str, latest: str):
    split_current_semver = current.split(".", 3)
    split_latest_semver = latest.split(".", 3)
    if split_current_semver[0] != split_latest_semver[0]:
      type="major"
      if VERSIONING == "major":
        return True
    if split_current_semver[1] != split_latest_semver[1]:
      type="minor"
      if VERSIONING != "patch":
        return True
    if split_current_semver[2] != split_latest_semver[2]:
      type="patch"
      return True
    return False
    

def execute_upgrades():
    if CATALOG == "ALL":
      filtered = filter(lambda a: a.update_available and a.status == "active", INSTALLED_CHARTS)
    else:
      filtered = filter(lambda a: a.update_available and a.status == "active" and a.catalog == CATALOG, INSTALLED_CHARTS)
    for chart in filtered:
      pre_update_ver = chart.human_version
      post_update_ver = chart.human_latest_version
      split_current_version = chart.human_version.split("_", 1)
      current_version = split_current_version[1]
      split_latest = chart.human_latest_version.split("_", 1)
      latest_version = split_latest[1]
      if check_semver(current_version, latest_version):
        print(f"Updating {chart.name}... \n")
        pre_update_ver = chart.human_version
        result = subprocess.run(['cli', '-c', f'app chart_release upgrade release_name="{chart.name}"'], capture_output=True)
        post_update_ver = chart.human_latest_version
        if "Upgrade complete" not in result.stdout.decode('utf-8'):
            print(f"{chart.name} failed to upgrade. \n{result.stdout.decode('utf-8')}")
        else:
            print(f"{chart.name} upgraded ({pre_update_ver} --> {post_update_ver})")

def fetch_charts():
  rawcharts = subprocess.run(["cli", "-c", "app chart_release query"], stdout=subprocess.PIPE)
  charts = rawcharts.stdout.decode('utf-8')
  return(charts)
  
def process_args():
    global CATALOG
    global VERSIONING
    global SYNC
    global PRUNE
    parser = argparse.ArgumentParser(description='Update TrueNAS SCALE Apps')
    parser.add_argument('--catalog', nargs='?', default='ALL', help='name of the catalog you want to process in caps. Or "ALL" to render all catalogs.')
    parser.add_argument('--versioning', nargs='?', default='minor', help='Name of the versioning scheme you want to update. Options: major, minor or patch. Defaults to minor')
    parser.add_argument('-s', '--sync', action="store_true", help='sync catalogs before trying to update')
    parser.add_argument('-p', '--prune', action="store_true", help='prune old docker images after update')
    args = parser.parse_args()
    CATALOG = args.catalog
    VERSIONING = args.versioning
    if args.sync:
      SYNC = True
    else:
      SYNC = False
    if args.prune:
      PRUNE = True
    else:
      PRUNE = False
    
def sync_catalog():
    if SYNC:
      print("Syncing Catalogs...\n")
      process = subprocess.Popen(["cli", "-c", "app catalog sync_all"], stdout=subprocess.PIPE)
      while process.poll() is None:
          lines = process.stdout.readline()
          print (lines)
      print (process.stdout.read())
  
def docker_prune():
    if PRUNE:
      print("Pruning old docker images...\n")
      process = subprocess.Popen(["docker", "image ", "prune", "-af"], stdout=subprocess.PIPE)
      while process.poll() is None:
          lines = process.stdout.readline()
          print (lines)
      print (process.stdout.read())
  
def run():
    process_args()
    print("Starting TrueCharts App updater...\n")
    sync_catalog()
    charts = fetch_charts()
    parse_charts(charts)
    print("Executing Updates...\n")
    execute_upgrades()
    print("Updating Finished\n")
    docker_prune()
    exit(0)


if __name__ == '__main__':
    run()

